@@ -19,6 +19,7 @@
 # Boston, MA 02111-1307, USA.
 #
 
+from .message import Position
 from .odict import odict
 from .utils import to_underscores
 
@@ -93,6 +94,11 @@ in contrast to the other create_type() functions."""
         elif gtype_name in ('GArray', 'GPtrArray', 'GByteArray'):
             return Array('GLib.' + gtype_name[1:], TYPE_ANY,
                          gtype_name=gtype_name)
+        elif gtype_name == 'GStrv':
+            bare_utf8 = TYPE_STRING.clone()
+            bare_utf8.ctype = None
+            return Array(None, bare_utf8, ctype=None, gtype_name=gtype_name,
+                         is_const=False)
         return cls(gtype_name=gtype_name)
 
     def get_giname(self):
@@ -487,12 +493,12 @@ GIName.  It's possible for nodes to contain or point to other nodes."""
     def inherit_file_positions(self, node):
         self.file_positions.update(node.file_positions)
 
-    def add_file_position(self, filename, line, column):
-        self.file_positions.add((filename, line, column))
+    def add_file_position(self, position):
+        self.file_positions.add(position)
 
     def add_symbol_reference(self, symbol):
         if symbol.source_filename:
-            self.add_file_position(symbol.source_filename, symbol.line, -1)
+            self.add_file_position(Position(symbol.source_filename, symbol.line))
 
     def walk(self, callback, chain):
         res = callback(self, chain)
@@ -765,6 +771,7 @@ class Field(Annotated):
         self.writable = writable
         self.bits = bits
         self.anonymous_node = anonymous_node
+        self.private = False
 
     def __cmp__(self, other):
         return cmp(self.name, other.name)
