@@ -22,13 +22,14 @@ import re
 from . import ast
 from . import message
 from .annotationparser import (TAG_VFUNC, TAG_SINCE, TAG_DEPRECATED, TAG_RETURNS,
-                               TAG_ATTRIBUTES, TAG_RENAME_TO, TAG_TYPE, TAG_TRANSFER,
+                               TAG_ATTRIBUTES, TAG_RENAME_TO, TAG_TYPE,
                                TAG_UNREF_FUNC, TAG_REF_FUNC, TAG_SET_VALUE_FUNC,
                                TAG_GET_VALUE_FUNC)
-from .annotationparser import (OPT_ALLOW_NONE,
-                               OPT_ARRAY, OPT_ELEMENT_TYPE, OPT_IN, OPT_INOUT,
+from .annotationparser import (OPT_ALLOW_NONE, OPT_ARRAY, OPT_ATTRIBUTE,
+                               OPT_ELEMENT_TYPE, OPT_IN, OPT_INOUT,
                                OPT_INOUT_ALT, OPT_OUT, OPT_SCOPE,
-                               OPT_TYPE, OPT_CLOSURE, OPT_DESTROY, OPT_SKIP,
+                               OPT_OUT_CALLER_ALLOCATES, OPT_OUT_CALLEE_ALLOCATES,
+                               OPT_TYPE, OPT_CLOSURE, OPT_DESTROY, OPT_TRANSFER, OPT_SKIP,
                                OPT_FOREIGN, OPT_ARRAY_FIXED_SIZE,
                                OPT_ARRAY_LENGTH, OPT_ARRAY_ZERO_TERMINATED)
 from .annotationparser import AnnotationParser
@@ -46,14 +47,12 @@ class MainTransformer(object):
     # Public API
 
     def transform(self):
-        # Dirty hack for now...maybe eventually we'll support the "typedef GSList FooSet"
-        # pattern.
-        if self._namespace.name == 'Atk':
-            attribute = self._namespace.get('Attribute')
-            attributeset = self._namespace.get('AttributeSet')
-            if attribute and attributeset:
-                alias = ast.Alias('AttributeSet', target=ast.TYPE_ANY)
-                self._namespace.append(alias, replace=True)
+        contents = list(self._namespace.itervalues())
+        if len(contents) == 0:
+            message.fatal("""Namespace is empty; likely causes are:
+* Not including .h files to be scanned
+* Broken --identifier-prefix
+""")
 
         # Some initial namespace surgery
         self._namespace.walk(self._pass_fixup_hidden_fields)
@@ -63,6 +62,9 @@ class MainTransformer(object):
         # Type() types and see if they match up with something.
         self._namespace.walk(self._pass_type_resolution)
 
+        # Read in annotations needed early
+        self._namespace.walk(self._pass_read_annotations_early)
+
         # Determine some default values for transfer etc.
         # based on the current tree.
         self._namespace.walk(self._pass_callable_defaults)
@@ -109,6 +111,8 @@ usage is void (*_gtk_reserved1)(void);"""
                                  ast.Record, ast.Union)):
             return True
         for field in node.fields:
+            if field is None:
+                continue
             if (field.name.startswith('_')
                 and field.anonymous_node is not None
                 and isinstance(field.anonymous_node, ast.Callback)):
@@ -164,6 +168,15 @@ usage is void (*_gtk_reserved1)(void);"""
         block = self._blocks.get(node.symbol)
         self._apply_annotations_callable(node, chain, block)
 
+    def _pass_read_annotations_early(self, node, chain):
+        if isinstance(node, ast.Record):
+            if node.ctype is not None:
+                block = self._blocks.get(node.ctype)
+            else:
+                block = self._blocks.get(node.c_name)
+            self._apply_annotations_annotated(node, block)
+        return True
+
     def _pass_callable_defaults(self, node, chain):
         if isinstance(node, (ast.Callable, ast.Signal)):
             for param in node.parameters:
@@ -173,25 +186,35 @@ usage is void (*_gtk_reserved1)(void);"""
                 node.retval.transfer = self._get_transfer_default(node, node.retval)
         return True
 
+    def _get_annotation_name(self, node):
+        if isinstance(node, (ast.Class, ast.Interface, ast.Record,
+                             ast.Union, ast.Enum, ast.Bitfield,
+                             ast.Callback, ast.Alias)):
+            if node.ctype is not None:
+                return node.ctype
+            elif isinstance(node, ast.Registered) and node.gtype_name is not None:
+                return node.gtype_name
+            return node.c_name
+        raise AssertionError("Unhandled node %r" % (node, ))
+
+    def _get_block(self, node):
+        return self._blocks.get(self._get_annotation_name(node))
+
     def _pass_read_annotations(self, node, chain):
         if not node.namespace:
             return False
+        if isinstance(node, ast.Alias):
+            self._apply_annotations_alias(node, chain)
         if isinstance(node, ast.Function):
             self._apply_annotations_function(node, chain)
         if isinstance(node, ast.Callback):
-            block = self._blocks.get(node.c_name)
-            self._apply_annotations_callable(node, chain, block)
-        if isinstance(node, (ast.Class, ast.Interface, ast.Record,
-                             ast.Union, ast.Enum, ast.Bitfield,
-                             ast.Callback)):
-            if node.ctype is not None:
-                block = self._blocks.get(node.ctype)
-            else:
-                block = self._blocks.get(node.c_name)
-            self._apply_annotations_annotated(node, block)
+            self._apply_annotations_callable(node, chain, block = self._get_block(node))
+        if isinstance(node, (ast.Class, ast.Interface, ast.Union, ast.Enum,
+                             ast.Bitfield, ast.Callback)):
+            self._apply_annotations_annotated(node, self._get_block(node))
         if isinstance(node, (ast.Class, ast.Interface, ast.Record, ast.Union)):
+            block = self._get_block(node)
             for field in node.fields:
-                self._blocks.get('%s::%s' % (node.c_name, field.name))
                 self._apply_annotations_field(node, block, field)
         if isinstance(node, (ast.Class, ast.Interface)):
             for prop in node.properties:
@@ -199,7 +222,7 @@ usage is void (*_gtk_reserved1)(void);"""
             for sig in node.signals:
                 self._apply_annotations_signal(node, sig)
         if isinstance(node, ast.Class):
-            block = self._blocks.get(node.c_name)
+            block = self._get_block(node)
             if block:
                 tag = block.get(TAG_UNREF_FUNC)
                 node.unref_func = tag.value if tag else None
@@ -220,7 +243,7 @@ usage is void (*_gtk_reserved1)(void);"""
         elif has_element_type:
             self._apply_annotations_element_type(parent, node, options)
 
-    def _resolve(self, type_str, orig_node=None):
+    def _resolve(self, type_str, type_node=None, node=None, parent=None):
         def grab_one(type_str, resolver, top_combiner, combiner):
             """Return a complete type, and the trailing string part after it.
             Use resolver() on each identifier, and combiner() on the parts of
@@ -250,16 +273,41 @@ usage is void (*_gtk_reserved1)(void);"""
                 "Too many parameters in type specification %r" % (type_str, ))
             return base
         def top_combiner(base, *rest):
-            if orig_node is not None:
-                base.is_const = orig_node.is_const
+            if type_node is not None and isinstance(type_node, ast.Type):
+                base.is_const = type_node.is_const
             return combiner(base, *rest)
 
         result, rest = grab_one(type_str, resolver, top_combiner, combiner)
         if rest:
             message.warn("Trailing components in type specification %r" % (
                 type_str, ))
+
+        if not result.resolved:
+            position = None
+            if parent is not None and isinstance(parent, ast.Function):
+                text = parent.symbol
+                position = self._get_position(parent, node)
+            else:
+                text = type_str
+            message.warn_node(parent, "%s: Unknown type: %r" %
+                              (text, result.ctype), positions=position)
         return result
 
+    def _get_position(self, func, param):
+        block = self._blocks.get(func.symbol)
+        if block:
+            if isinstance(param, ast.Parameter):
+                tag = block.tags.get(param.argname)
+            elif isinstance(param, ast.Return):
+                tag = block.tags.get(TAG_RETURNS)
+            else:
+                tag = None
+
+            if tag.position:
+                return tag.position
+
+        return block.position
+
     def _apply_annotations_array(self, parent, node, options):
         array_opt = options.get(OPT_ARRAY)
         if array_opt:
@@ -269,7 +317,8 @@ usage is void (*_gtk_reserved1)(void);"""
 
         element_type = options.get(OPT_ELEMENT_TYPE)
         if element_type is not None:
-            element_type_node = self._resolve(element_type.one())
+            element_type_node = self._resolve(element_type.one(),
+                                              node.type, node, parent)
         elif isinstance(node.type, ast.Array):
             element_type_node = node.type.element_type
         else:
@@ -300,21 +349,52 @@ usage is void (*_gtk_reserved1)(void);"""
                 container_type.length_param_name = param.argname
         fixed = array_values.get(OPT_ARRAY_FIXED_SIZE)
         if fixed:
-            container_type.size = int(fixed)
+            try:
+                container_type.size = int(fixed)
+            except ValueError:
+                # Already warned in annotationparser.py
+                return
         node.type = container_type
 
     def _apply_annotations_element_type(self, parent, node, options):
         element_type_opt = options.get(OPT_ELEMENT_TYPE)
-        element_type = element_type_opt.flat()
+        if element_type_opt is None:
+            message.warn(
+                'element-type annotation takes at least one option, '
+                'none given',
+                options.position)
+            return
+
         if isinstance(node.type, ast.List):
-            assert len(element_type) == 1
-            node.type.element_type = self._resolve(element_type[0])
+            if element_type_opt.length() != 1:
+                message.warn(
+                    'element-type annotation for a list must have exactly '
+                    'one option, not %d options' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            node.type.element_type = self._resolve(element_type_opt.one(),
+                                                   node.type, node, parent)
         elif isinstance(node.type, ast.Map):
-            assert len(element_type) == 2
-            node.type.key_type = self._resolve(element_type[0])
-            node.type.value_type = self._resolve(element_type[1])
+            if element_type_opt.length() != 2:
+                message.warn(
+                    'element-type annotation for a hash table must have exactly '
+                    'two options, not %d option(s)' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            element_type = element_type_opt.flat()
+            node.type.key_type = self._resolve(element_type[0],
+                                               node.type, node, parent)
+            node.type.value_type = self._resolve(element_type[1],
+                                                 node.type, node, parent)
         elif isinstance(node.type, ast.Array):
-            node.type.element_type = self._resolve(element_type[0])
+            if element_type_opt.length() != 1:
+                message.warn(
+                    'element-type annotation for an array must have exactly '
+                    'one option, not %d options' % (element_type_opt.length(), ),
+                    options.position)
+                return
+            node.type.element_type = self._resolve(element_type_opt.one(),
+                                                   node.type, node, parent)
         else:
             message.warn_node(parent,
                 "Unknown container %r for element-type annotation" % (node.type, ))
@@ -362,7 +442,8 @@ usage is void (*_gtk_reserved1)(void);"""
         if isinstance(target, ast.Alias):
             return self._get_transfer_default_returntype_basic(target.target)
         elif (isinstance(target, ast.Boxed)
-              or (isinstance(target, (ast.Record, ast.Union)) and target.gtype_name is not None)):
+              or (isinstance(target, (ast.Record, ast.Union))
+                  and (target.gtype_name is not None or target.foreign))):
             return ast.PARAM_TRANSFER_FULL
         elif isinstance(target, (ast.Enum, ast.Bitfield)):
             return ast.PARAM_TRANSFER_NONE
@@ -378,7 +459,7 @@ usage is void (*_gtk_reserved1)(void);"""
             elif isinstance(target, (ast.Record, ast.Union)):
                 return ast.PARAM_TRANSFER_FULL
             else:
-                assert False, "Invalid constructor"
+                raise AssertionError("Invalid constructor")
         elif isinstance(target, (ast.Class, ast.Record, ast.Union)):
             # Explicitly no default for these
             return None
@@ -404,7 +485,8 @@ usage is void (*_gtk_reserved1)(void);"""
 
         param_type = options.get(OPT_TYPE)
         if param_type:
-            node.type = self._resolve(param_type.one(), node.type)
+            node.type = self._resolve(param_type.one(),
+                                      node.type, node, parent)
 
         caller_allocates = False
         annotated_direction = None
@@ -418,16 +500,16 @@ usage is void (*_gtk_reserved1)(void);"""
             annotated_direction = ast.PARAM_DIRECTION_OUT
             if subtype in (None, ''):
                 if node.type.target_giname and node.type.ctype:
-                    caller_allocates = '**' not in node.type.ctype
+                    target = self._transformer.lookup_giname(node.type.target_giname)
+                    has_double_indirection = '**' in node.type.ctype
+                    is_structure_or_union = isinstance(target, (ast.Record, ast.Union))
+                    caller_allocates = (not has_double_indirection and is_structure_or_union)
                 else:
                     caller_allocates = False
-            elif subtype == 'caller-allocates':
+            elif subtype == OPT_OUT_CALLER_ALLOCATES:
                 caller_allocates = True
-            elif subtype == 'callee-allocates':
+            elif subtype == OPT_OUT_CALLEE_ALLOCATES:
                 caller_allocates = False
-            else:
-                message.fatal("out allocation for %s is invalid (%r)" % (
-                    node, subtype))
         elif OPT_IN in options:
             annotated_direction = ast.PARAM_DIRECTION_IN
 
@@ -437,8 +519,8 @@ usage is void (*_gtk_reserved1)(void);"""
             # Also reset the transfer default if we're toggling direction
             node.transfer = self._get_transfer_default(parent, node)
 
-        transfer_tag = options.get(TAG_TRANSFER)
-        if transfer_tag:
+        transfer_tag = options.get(OPT_TRANSFER)
+        if transfer_tag and transfer_tag.length() == 1:
             node.transfer = transfer_tag.one()
 
         self._adjust_container_type(parent, node, options)
@@ -450,11 +532,9 @@ usage is void (*_gtk_reserved1)(void);"""
         if tag is not None and tag.comment is not None:
             node.doc = tag.comment
 
-        for key in options:
-            if '.' in key:
-                value = options.get(key)
-                if value:
-                    node.attributes.append((key, value.one()))
+        if options:
+            for attribute in options.getall(OPT_ATTRIBUTE):
+                node.attributes.append(attribute.flat())
 
     def _apply_annotations_annotated(self, node, block):
         if block is None:
@@ -480,7 +560,7 @@ usage is void (*_gtk_reserved1)(void);"""
 
         annos_tag = block.get(TAG_ATTRIBUTES)
         if annos_tag is not None:
-            options = AnnotationParser.parse_options(annos_tag.value)
+            options = AnnotationParser.parse_options(annos_tag, annos_tag.value)
             for key, value in options.iteritems():
                 if value:
                     node.attributes.append((key, value.one()))
@@ -491,6 +571,10 @@ usage is void (*_gtk_reserved1)(void);"""
         if OPT_FOREIGN in block.options:
             node.foreign = True
 
+    def _apply_annotations_alias(self, node, chain):
+        block = self._get_block(node)
+        self._apply_annotations_annotated(node, block)
+
     def _apply_annotations_param(self, parent, param, tag):
         if tag:
             options = tag.options
@@ -498,17 +582,9 @@ usage is void (*_gtk_reserved1)(void);"""
             options = {}
         if isinstance(parent, (ast.Function, ast.VFunction)):
             scope = options.get(OPT_SCOPE)
-            if scope:
-                scope = scope.one()
-                if scope not in [ast.PARAM_SCOPE_CALL,
-                                 ast.PARAM_SCOPE_ASYNC,
-                                 ast.PARAM_SCOPE_NOTIFIED]:
-                    message.warn_node(
-                        parent,
-                        "Invalid scope %r for parameter %r" % (scope, param.argname))
-                else:
-                    param.scope = scope
-                    param.transfer = ast.PARAM_TRANSFER_NONE
+            if scope and scope.length() == 1:
+                param.scope = scope.one()
+                param.transfer = ast.PARAM_TRANSFER_NONE
 
             destroy = options.get(OPT_DESTROY)
             if destroy:
@@ -523,7 +599,7 @@ usage is void (*_gtk_reserved1)(void);"""
                     # since we don't have a way right now to flag this callback a destroy.
                     destroy_param.scope = ast.PARAM_SCOPE_NOTIFIED
             closure = options.get(OPT_CLOSURE)
-            if closure:
+            if closure and closure.length() == 1:
                 param.closure_name = self._get_validate_parameter_name(parent,
                                                                        closure.one(),
                                                                        param)
@@ -545,12 +621,39 @@ usage is void (*_gtk_reserved1)(void);"""
         self._apply_annotations_param_ret_common(parent, return_, tag)
 
     def _apply_annotations_params(self, parent, params, block):
+        allparams = []
+        if parent.instance_parameter:
+            allparams.append(parent.instance_parameter.argname)
         for param in params:
             if block:
                 tag = block.get(param.argname)
             else:
                 tag = None
             self._apply_annotations_param(parent, param, tag)
+            allparams.append(param.argname)
+
+        if not block:
+            return
+        docparams = block.params[:]
+        for doc_name in docparams:
+            if doc_name in allparams:
+                continue
+            # Skip varargs, see #629759
+            if doc_name.lower() in ['...', 'varargs', TAG_RETURNS]:
+                continue
+            if len(allparams) == 0:
+                text = ''
+            elif len(allparams) == 1:
+                text = ', should be %r' % (allparams[0], )
+            else:
+                text = ', should be one of %s' % (
+                ', '.join(repr(p) for p in allparams), )
+
+            tag = block.get(doc_name)
+            message.warn(
+                '%s: unknown parameter %r in documentation comment%s' % (
+                block.name, doc_name, text),
+                tag.position)
 
     def _apply_annotations_callable(self, node, chain, block):
         self._apply_annotations_annotated(node, block)
@@ -577,43 +680,49 @@ usage is void (*_gtk_reserved1)(void);"""
         tag = block.get(field.name)
         if not tag:
             return
-        t = tag.options.get('type')
+        t = tag.options.get(OPT_TYPE)
         if not t:
             return
         field.type = self._transformer.create_type_from_user_string(t.one())
 
     def _apply_annotations_property(self, parent, prop):
-        block = self._blocks.get('%s:%s' % (parent.c_name, prop.name))
+        prefix = self._get_annotation_name(parent)
+        block = self._blocks.get('%s:%s' % (prefix, prop.name))
         self._apply_annotations_annotated(prop, block)
         if not block:
             return
-        transfer_tag = block.get(TAG_TRANSFER)
+        transfer_tag = block.get(OPT_TRANSFER)
         if transfer_tag is not None:
             prop.transfer = transfer_tag.value
         else:
             prop.transfer = self._get_transfer_default(parent, prop)
         type_tag = block.get(TAG_TYPE)
         if type_tag:
-            prop.type = self._resolve(type_tag.value, prop.type)
+            prop.type = self._resolve(type_tag.value, prop.type, prop, parent)
 
     def _apply_annotations_signal(self, parent, signal):
-        block = self._blocks.get('%s::%s' % (parent.c_name, signal.name))
+        prefix = self._get_annotation_name(parent)
+        block = self._blocks.get('%s::%s' % (prefix, signal.name))
         self._apply_annotations_annotated(signal, block)
         # We're only attempting to name the signal parameters if
         # the number of parameter tags (@foo) is the same or greater
         # than the number of signal parameters
         if block and len(block.tags) > len(signal.parameters):
             names = block.tags.items()
+            # Resolve real parameter names early, so that in later
+            # phase we can refer to them while resolving annotations.
+            for i, param in enumerate(signal.parameters):
+                param.argname, tag = names[i+1]
         else:
             names = []
         for i, param in enumerate(signal.parameters):
             if names:
                 name, tag = names[i+1]
-                param.name = name
                 options = getattr(tag, 'options', {})
                 param_type = options.get(OPT_TYPE)
                 if param_type:
-                    param.type = self._resolve(param_type.one(), param.type)
+                    param.type = self._resolve(param_type.one(), param.type,
+                                               param, parent)
             else:
                 tag = None
             self._apply_annotations_param(signal, param, tag)
@@ -799,16 +908,29 @@ method or constructor of some type."""
 
         # A quick hack here...in the future we should catch C signature/GI signature
         # mismatches in a general way in finaltransformer
-        if first.type.ctype.count('*') != 1:
+        if first.type.ctype is not None and first.type.ctype.count('*') != 1:
             return False
 
-        uscored = self._uscored_identifier_for_type(first.type)
-        if not subsymbol.startswith(uscored):
-            return False
-        del func.parameters[0]
+        # Here we check both the c_symbol_prefix and (if that fails),
+        # attempt to do a default uscoring of the type.  The reason we
+        # look at a default underscore transformation is for
+        # gdk_window_object_get_type(), which says to us that the
+        # prefix is "gdk_window_object", when really it's just
+        # "gdk_window".  Possibly need an annotation to override this.
+        prefix_matches = False
+        uscored_prefix = None
+        if hasattr(target, 'c_symbol_prefix') and target.c_symbol_prefix is not None:
+            prefix_matches = subsymbol.startswith(target.c_symbol_prefix)
+            if prefix_matches:
+                uscored_prefix = target.c_symbol_prefix
+        if not prefix_matches:
+            uscored_prefix = self._uscored_identifier_for_type(first.type)
+            if not subsymbol.startswith(uscored_prefix):
+                return False
+        func.instance_parameter = func.parameters.pop(0)
         subsym_idx = func.symbol.find(subsymbol)
         self._namespace.float(func)
-        func.name = func.symbol[(subsym_idx + len(uscored) + 1):]
+        func.name = func.symbol[(subsym_idx + len(uscored_prefix) + 1):]
         target.methods.append(func)
         func.is_method = True
         return True
@@ -937,6 +1059,7 @@ method or constructor of some type."""
             if matched_signal:
                 continue
             vfunc = ast.VFunction.from_callback(callback)
+            vfunc.instance_parameter = callback.parameters[0]
             vfunc.inherit_file_positions(callback)
             node.virtual_methods.append(vfunc)
 
